import {
  Alert,
  Box,
  Button,
  Modal,
  TextField,
  Typography,
} from '@suid/material';
import { Component, createEffect, createSignal, Show } from 'solid-js';
import { parseUserId } from '../../lib/user-id';
import dispatch from '../../workers/dispatcher-main';
import Dialog from '../dialog';

interface props {}

const [invitation, setInvitation] = createSignal<any>(false);

const fetchInvitation = () => {
  fetch(invitation().url)
    .then((response) => response.json())
    .then((data) => {
      setInvitation({ ...invitation(), data });
      return invitation();
    })
    .then(() => {
      dispatch({
        action: 'getAccount',
        params: { id: invitation().data.id },
      }).then((account) => {
        console.log({ caller: 'Invitation / fetchInvitation', account });
        setInvitation({ ...invitation(), existing: true });
      });
    })
    .catch((error) => {
      console.error({ caller: 'Invitation / fetchInvitation', error });
    });
};

export const searchInvitationInIntent = (intent: any) => {
  if (intent.action !== 'android.intent.action.VIEW') {
    return false;
  }
  let url;
  try {
    url = new URL(intent.data);
  } catch (error) {
    console.error({ caller: 'searchInvitationInIntent', intent, error });
    return false;
  }
  if (url.protocol !== 'geo:') {
    return false;
  }
  const q = url.search;
  const matches = q.match(/invitation=(.+)/);
  if (!matches) {
    return false;
  }
  try {
    const payload = matches[1];
    const decoded = atob(payload);
    const url = new URL(decoded);
    setInvitation({ url });
    fetchInvitation();
  } catch {
    console.warn({
      caller: 'Invitation / searchInvitationInIntent',
      intent,
      matches,
      message: 'Query is not a base64 encoded URL',
    });
  }
  return true;
};

export const searchInvitation = () => {
  const hash = location.hash;
  console.log({ caller: 'Invitation / searchInvitation', hash });

  if (hash.length > 1) {
    try {
      const payload = hash.slice(1);
      const decoded = atob(payload);
      const url = new URL(decoded);
      setInvitation({ url });
      fetchInvitation();
    } catch {
      console.warn({
        caller: 'Invitation / searchInvitation',
        hash,
        message: 'Hash is not a base64 encoded URL',
      });
    }
  }
};

const Invitation: Component<props> = (props) => {
  createEffect(() => {
    console.log({
      caller: 'Invitation / createEffect',
      invitation: invitation(),
    });
  });

  const closeHandler = () => {};
  const submitHandler = async () => {
    try {
      const response = await fetch(
        `${invitation().url.href}/${invitation().code}`
      );
      const codeData = await response.json();
      setInvitation({
        ...invitation(),
        codeData,
        failure: codeData.response !== 'OK',
        success: codeData.response === 'OK',
      });
      const user = invitation().codeData.user;
      const id = user._id;
      const account = {
        id,
        localDb: id,
        name: id,
        remoteDbServer: user.database,
        remoteDbUser: user.username,
        remoteDbPassword: user.password,
        mail: user.mail,
      };
      dispatch({ action: 'putAccount', params: { id, account } });
      const settings: any = await dispatch({ action: 'getSettings' });
      dispatch({
        action: 'putSettings',
        params: { settings: { ...settings, currentAccountId: id } },
      });
    } catch (error) {
      setInvitation({
        ...invitation(),
        failure: true,
      });
    }
  };

  const codeChangeHandler = (event: any) => {
    setInvitation({ ...invitation(), code: event.target.value });
  };

  const handleModalFailureClose = () => {
    setInvitation({ ...invitation(), failure: undefined, existing: undefined });
  };

  const handleCloseSuccess = () => {
    setInvitation(false);
    location.reload();
  };

  return (
    <>
      <Dialog
        open={!!invitation() && !!invitation().data}
        fullScreen={false}
        closeHandler={closeHandler}
        title='Invitation'
      >
        <Box
          component='form'
          sx={{
            width: '100%',
            '& .MuiTextField-root': { m: 1, width: '100%' },
            paddingTop: '5px',
          }}
          noValidate
          autoComplete='off'
        >
          <p>
            Pour configurer cette application en tant que "
            {invitation().data.id}
            ", copier le code qui vous a été envoyé par mail à l'adresse "
            {invitation().data.mail}" :{' '}
          </p>
          <TextField
            required
            label='Code'
            type='number'
            onChange={codeChangeHandler}
          />
          <Button variant='contained' onClick={submitHandler}>
            OK
          </Button>
          <Button variant='outlined' onClick={fetchInvitation}>
            Renvoyer le code
          </Button>
          <Button variant='outlined' color='error' onClick={handleCloseSuccess}>
            Annuler
          </Button>
        </Box>
      </Dialog>
      <Modal
        open={!!invitation().failure}
        onClose={handleModalFailureClose}
        aria-labelledby='modal-modal-title'
        aria-describedby='modal-modal-description'
      >
        <Box
          sx={{
            position: 'absolute',
            top: '50%',
            left: '50%',
            transform: 'translate(-50%, -50%)',
            width: 400,
            border: '2px solid #000',
            boxShadow: '24px',
            p: 4,
          }}
        >
          <Alert severity='error'>Le code est erroné !</Alert>
        </Box>
      </Modal>{' '}
      <Modal
        open={!!invitation().success}
        onClose={handleCloseSuccess}
        aria-labelledby='modal-modal-title'
        aria-describedby='modal-modal-description'
      >
        <Box
          sx={{
            position: 'absolute',
            top: '50%',
            left: '50%',
            transform: 'translate(-50%, -50%)',
            width: 400,
            border: '2px solid #000',
            boxShadow: '24px',
            p: 4,
          }}
        >
          <Alert severity='success'>C'est tout bon !</Alert>
        </Box>
      </Modal>{' '}
      <Modal
        open={!!invitation().existing}
        onClose={handleModalFailureClose}
        aria-labelledby='modal-modal-title'
        aria-describedby='modal-modal-description'
      >
        <Box
          sx={{
            position: 'absolute',
            top: '50%',
            left: '50%',
            transform: 'translate(-50%, -50%)',
            width: 400,
            border: '2px solid #000',
            boxShadow: '24px',
            p: 4,
          }}
        >
          <Alert severity='info'>
            Ce compte est déjà configuré, vous pouvez soit continuer et le
            reconfigurer soit abandonner cette action et ne pas tenir compte du
            message qui vous a été envoyé.
          </Alert>
        </Box>
      </Modal>
    </>
  );
};

export default Invitation;
