import { Component, createEffect, onMount } from 'solid-js';
import OlMap from 'ol/Map';

import {
  currentOverlayDefinition,
  currentOverlayHighlightedDefinition,
  isHighlighted,
} from '../map-tile-provider';
import { Feature } from 'ol';
import MVT from 'ol/format/MVT';
import { Point } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import VectorTileLayer from 'ol/layer/VectorTile';
import VectorTileSource from 'ol/source/VectorTile.js';
import WMTSTileGrid from 'ol/tilegrid/WMTS.js';
import Cluster from 'ol/source/Cluster';
import { style } from '../gpx/styles';
import ClusterableVectorTileSourceProxy from '../../lib/ClusterableVectorTileSourceProxy';
import { getMap, getZoomInteger } from '../map/Map';
import { getCenter } from 'ol/extent';
import { Projection } from 'ol/proj';
import { getVectorTileFeatureType } from './overlay-definitions';
import GeoJSON from 'ol/format/GeoJSON.js';
import TopoJSON from 'ol/format/TopoJSON.js';

interface Props {
  map: () => OlMap | null;
}

let clusterLayer: VectorLayer;
let vectorTileLayer: VectorTileLayer;

export const Overlays: Component<Props> = ({ map }) => {
  onMount(() => {
    const gridsetName = 'EPSG:900913';
    const gridNames = [
      'EPSG:900913:0',
      'EPSG:900913:1',
      'EPSG:900913:2',
      'EPSG:900913:3',
      'EPSG:900913:4',
      'EPSG:900913:5',
      'EPSG:900913:6',
      'EPSG:900913:7',
      'EPSG:900913:8',
      'EPSG:900913:9',
      'EPSG:900913:10',
      'EPSG:900913:11',
      'EPSG:900913:12',
      'EPSG:900913:13',
      'EPSG:900913:14',
    ];
    const baseUrl = 'https://geo.dyomedea.com/geoserver/gwc/service/wmts';
    const geoServerStyle = '';
    const format = 'application/vnd.mapbox-vector-tile';
    const infoFormat = 'text/html';
    const layerName = 'dyomedea';
    // const projection = new Projection({
    //   code: 'EPSG:900913',
    //   units: 'm',
    //   axisOrientation: 'neu',
    // });
    // const resolutions = [
    //   156543.03390625, 78271.516953125, 39135.7584765625, 19567.87923828125,
    //   9783.939619140625, 4891.9698095703125, 2445.9849047851562,
    //   1222.9924523925781, 611.4962261962891, 305.74811309814453,
    //   152.87405654907226, 76.43702827453613, 38.218514137268066,
    //   19.109257068634033, 9.554628534317017,
    // ];

    // const params = {
    //   REQUEST: 'GetTile',
    //   SERVICE: 'WMTS',
    //   VERSION: '1.0.0',
    //   LAYER: layerName,
    //   STYLE: geoServerStyle,
    //   TILEMATRIX: gridsetName + ':{z}',
    //   TILEMATRIXSET: gridsetName,
    //   FORMAT: format,
    //   TILECOL: '{x}',
    //   TILEROW: '{y}',
    // };

    // function constructSource() {
    //   let url = baseUrl + '?';
    //   for (const param in params) {
    //     url = url + param + '=' + params[param] + '&';
    //   }
    //   url = url.slice(0, -1);

    //   const source = new VectorTileSource({
    //     url: url,
    //     format: new MVT({ featureClass: Feature }),
    //     projection: projection,
    //     // minZoom: 8,
    //     // maxZoom: 14,
    //     tileGrid: new WMTSTileGrid({
    //       tileSize: [256, 256],
    //       origin: [-2.003750834e7, 2.003750834e7],
    //       resolutions: resolutions,
    //       matrixIds: gridNames,
    //       minZoom: 10,
    //     }),
    //     wrapX: true,
    //   });
    //   return source;
    // }

    // http://localhost:8081/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER=dyomedea&STYLE=&TILEMATRIX=EPSG:900913:14&TILEMATRIXSET=EPSG:900913&FORMAT=application/vnd.mapbox-vector-tile&TILECOL=8262&TILEROW=6048
    // http://localhost:8081/geoserver/gwc/service/wmts?REQUEST=GetTile&SERVICE=WMTS&VERSION=1.0.0&LAYER=dyomedea&STYLE=&TILEMATRIX=EPSG:900913:14&TILEMATRIXSET=EPSG:900913&FORMAT=application/vnd.mapbox-vector-tile&TILECOL=8265&TILEROW=6046

    // const vectorTileSource = constructSource();

    const vectorTileSource = new VectorTileSource({
      url: 'https://geo.dyomedea.com/geoserver/gwc/service/wmts/rest/hiking/EPSG:900913/EPSG:900913:{z}/{y}/{x}?format=application/json;type=geojson',
      // url: 'https://geo.dyomedea.com/geoserver/gwc/service/wmts/rest/hiking/EPSG:900913/EPSG:900913:{z}/{y}/{x}?format=application/vnd.mapbox-vector-tile',
      //       url: 'http://localhost:8080/geoserver/gwc/service/tms/1.0.0/test%3Atransportation@EPSG%3A900913@pbf/{z}/{x}/{-y}.pbf',
      //      url: 'https://geo.dyomedea.com/services/spain/tiles/{z}/{x}/{y}.pbf',
      format: new GeoJSON({ dataProjection: 'EPSG:900913' }),
      //      format: new MVT({ featureClass: Feature }),
      maxZoom: 15,
      tileSize: 256,
    });

    vectorTileLayer = new VectorTileLayer({
      source: vectorTileSource,
      style,
      declutter: false,
    });

    const clusterableVectorTileSource = new ClusterableVectorTileSourceProxy({
      source: vectorTileSource,
    });

    const clusterSource = new Cluster({
      source: clusterableVectorTileSource,
      distance: 30,
      minDistance: 10,
      geometryFunction: (feature: Feature) => {
        // console.log({
        //   caller: 'Map / Cluster / geometryFunction ',
        //   feature,
        // });
        if (
          getZoomInteger() < 16 &&
          getVectorTileFeatureType(feature) === 'poi' &&
          isHighlighted(feature)
        ) {
          return new Point(getCenter(feature.getGeometry().getExtent()));
        }
        return null;
      },
      createCluster: (point: Point, features: Feature[]) => {
        // console.log({
        //   caller: 'Map / Cluster / createCluster',
        //   point,
        //   features,
        // });

        return new Feature({
          geometry: point,
          features: features,
          type: 'cluster',
        });
      },
    });

    clusterLayer = new VectorLayer({
      source: clusterSource,
      zIndex: Infinity,
      style,
    });
  });

  createEffect(() => {
    const overlayDefinition = currentOverlayDefinition();
    const overlayHighlightedDefinition = currentOverlayHighlightedDefinition();
    const olMap = getMap();
    if (olMap) {
      olMap.removeLayer(clusterLayer);
      olMap.removeLayer(vectorTileLayer);
      if (Object.keys(overlayDefinition).length > 0) {
        vectorTileLayer.changed();
        clusterLayer.changed();
        olMap.addLayer(vectorTileLayer);
        olMap.addLayer(clusterLayer);
      }
    }
  });

  return <></>;
};
